/* © SRSoftware 2024 */
package de.srsoftware.cal.api;

import java.util.Map;
import org.json.JSONObject;

/**
 * cartesian coords
 * @param latitude the latitude
 * @param longitude the longitude
 */
public record Coords(double latitude,double longitude) {
	/**
	 * get a string representing this coords in the ICAL format, see <a href="https://datatracker.ietf.org/doc/html/rfc5545#section-3.8.1.6">iCalendar spec</a>
	 * @return the formatted coords
	 */
	public String icalFormat(){
		return "%s;%s".formatted(latitude,longitude);
	}

	/**
	 * create a JSON object containing lat and lon of this coordinate
	 * @return the json object
	 */
	public JSONObject json() {
		return new JSONObject(Map.of("lat", latitude,"lon", longitude ));
	}

	@Override
	public String toString() {
		return "%s, %s".formatted(latitude,longitude);
	}
}
