/* © SRSoftware 2024 */
package de.srsoftware.cal.db;

import de.srsoftware.cal.api.Appointment;
import de.srsoftware.tools.Result;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.Set;

/**
 * Interface for calendar database
 */
public interface Database {
	/**
	 * add an appointment to the database
	 * @param appointment the appointment to store
	 * @return a clone of the provided appointment with id field set
	 */
	Result<Appointment> add(Appointment appointment);

	Result<List<String>> findTags(String infix);

	/**
	 * list appointments unfiltered
	 * @param count the maximum number of appointments to return
	 * @param offset the number of appointments to skip
	 * @param from restrict appointments to times after this date time
	 * @param till restrict appointments to times before this date time
	 * @return list of appointments in this time span
	 */
	Result<List<Appointment>> list(LocalDateTime from, LocalDateTime till, Integer count, Integer offset, Collection<String> tags);

	/**
	 * list appointments
	 * @param tags only list appointments which have matching tags
	 * @param count the maximum number of appointments to return
	 * @param offset the number of appointments to skip
	 * @return the list of appointments fetched from the db
	 */
	List<Appointment> listByTags(Set<String> tags, Integer count, Integer offset);

	Result<Appointment> loadEvent(long id);

	Result<Appointment> loadEvent(String location, LocalDateTime start);

	Result<Long> removeAppointment(long id);

	Result<Appointment> update(Appointment baseAppointment);
}
