/* © SRSoftware 2024 */
package de.srsoftware.cal.importer.jena;

import static de.srsoftware.cal.Util.*;
import static de.srsoftware.tools.Result.transform;
import static de.srsoftware.tools.Tag.CLASS;
import static de.srsoftware.tools.Tag.DIV;
import static de.srsoftware.tools.TagFilter.*;

import de.srsoftware.cal.BaseImporter;
import de.srsoftware.cal.api.Coords;
import de.srsoftware.tools.*;
import java.security.NoSuchAlgorithmException;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;
import java.util.function.Predicate;

public class Kassablanca extends BaseImporter {
	public static final String   BASE_URL	= "https://www.kassablanca.de";
	private static final Coords COORDS = new Coords(50.92093, 11.57788);
	private static final String  LOCATION	= "Kassablanca e.V., Felsenkellerstr. 13a, 07745 Jena";

	public Kassablanca() throws NoSuchAlgorithmException {
		super();
	}

	@Override
	protected String baseUrl() {
		return BASE_URL;
	}

	@Override
	public String description() {
		return "Importiert Events des Studentenclubs „Kassablanca“ in Jena";
	}

	@Override
	protected Predicate<Tag> extractAttachmentsFilter() {
		return attributeEquals(CLASS,"entry-content");
	}

	@Override
	protected Predicate<Tag> extractDescriptionFilter() {
		return attributeEquals(CLASS,"se-content");
	}

	@Override
	protected Result<Coords> extractCoords(Tag eventTag) {
		return Payload.of(COORDS);
	}

	@Override
	protected Predicate<Tag> extractEndDateFilter() {
		return null;
	}

	@Override
	protected Predicate<Tag> extractEndTimeFilter() {
		return null;
	}

	@Override
	protected Predicate<Tag> extractEventTagFilter() {
		return attributeEquals(CLASS,"entry-content");
	}

	@Override
	protected Result<List<String>> extractEventUrls(Result<Tag> programPage) {
		if (programPage.optional().isEmpty()) return transform(programPage);
		List<String> list = programPage.optional()
				.get()	//
				.find(attributeHas("class", "eventrow"))
				.stream()
				.flatMap(t -> t.find(ofType("h3")).stream())
				.map(t -> t.find(IS_ANCHOR))
				.flatMap(List::stream)
				.map(t -> t.get("href"))
				.toList();
		return Payload.of(list);
	}

	@Override
	protected Predicate<Tag> extractLinksFilter() {
		return attributeEquals(CLASS,"se-container");
	}

	@Override
	protected Result<Tag> extractLinksTag(Tag eventTag) {
		var top = eventTag.find(attributeEquals(CLASS,"se-container"));
		var bottom = eventTag.find(attributeEquals(CLASS, "se-content"));
		var common = Tag.of(DIV).addAll(top).addAll(bottom);
		return Payload.of(common);
	}

	@Override
	protected Result<String> extractLocation(Tag eventTag) {
		return Payload.of(LOCATION);
	}

	@Override
	protected Predicate<Tag> extractLocationFilter() {
		return null;
	}

	@Override
	protected Predicate<Tag> extractStartDateFilter() {
		return attributeEquals(CLASS,"se-header");
	}

	@Override
	protected Predicate<Tag> extractStartTimeFilter() {
		return attributeEquals(CLASS,"se-header");
	}

	@Override
	protected List<String> extractTags(Tag eventTag) {
		return List.of("Kassablanca","Jena");
	}

	@Override
	protected Predicate<Tag> extractTitleFilter() {
		return ofType("h1");
	}

	@Override
	protected Result<LocalDate> parseEndDate(String string) {
		return null;
	}

	@Override
	protected Result<LocalTime> parseEndTime(String string) {
		return null;
	}

	@Override
	protected Result<LocalDate> parseStartDate(String string) {
		return parseGermanDate(string);
	}

	@Override
	protected Result<LocalTime> parseStartTime(String string) {
		return parseGermanTime(string);
	}

	@Override
	protected String programURL() {
		return BASE_URL + "/programm";
	}
}
