/* © SRSoftware 2024 */
package de.srsoftware.cal;

import com.sun.net.httpserver.HttpExchange;
import de.srsoftware.tools.PathHandler;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.file.Path;
import java.util.Optional;

public class StaticHandler extends PathHandler {
	private final Optional<String> staticPath;

	public StaticHandler(Optional<String> staticPath) {
		this.staticPath = staticPath;
	}

	@Override
	public boolean doGet(String path, HttpExchange ex) throws IOException {
		if (!path.contains(".")) path += ".html";
		if (path.startsWith("/")) path = path.substring(1);
		var url = getClass().getClassLoader().getResource(path);
		if (staticPath.isPresent()) {
			var file = Path.of(staticPath.get()).resolve(path).toFile();
			if (file.exists() && file.isFile()) url = file.toURI().toURL();
		}
		if (url == null) return PathHandler.notFound(ex);
		var conn = url.openConnection();
		var mime = conn.getContentType();
		try (var input = conn.getInputStream()) {
			var bos = new ByteArrayOutputStream();
			input.transferTo(bos);
			ex.getResponseHeaders().add(PathHandler.CONTENT_TYPE, mime);
			return PathHandler.sendContent(ex, bos.toByteArray());
		}
	}
}
