var keyTimer = null;
var selectedTags = new Set();
var marker = null;

function fetchTags(text){
    if (text) fetch('/api/tags?infix='+text).then(handleTags);
}

function removeParent(target){
    var p = target.parentNode;
    p.parentNode.removeChild(p);
}

function addLink(evt){
    var input = element('link-url');
    var url = input.value;
    var name = prompt('Description for '+url,'Homepage');
    var span = document.createElement('span');
    span.innerHTML = `<a href="${url}">${name}</a><a onclick="removeParent(this)">❌</a> `;
    element('link-list').appendChild(span);
    input.value = '';
}

function addImage(evt){
    var input = element('image-url');
    var url = input.value;
    var span = document.createElement('span');
    span.innerHTML = `<img src="${url}" /><a onclick="removeParent(this)">❌</a> `;
    element('attachment-list').appendChild(span);
    input.value = '';

}

async function handleTags(response){
    if (response.ok){
        var tags = await response.json();
        if (tags.length == 0) return;
        var select = element('proposals');
        var input = element('tags-input');
        if (!select) return;
        select.innerHTML = '';
        tags.forEach(tag => {
            var option = document.createElement('option');
            option.value = tag;
            option.innerHTML = tag;
            option.onclick = e => {
                select.innerHTML = '';
                select.style.display = 'none';
                input.value = tag;
                input.focus();
            };
            select.appendChild(option);
            select.onkeyup = selectKeyPress;

        });
        select.style.display = 'block';
        select.style.height = (22*tags.length)+'px';
    }
}

function onMapClick(e) {
    element('coords').value = `${e.latlng.lat}, ${e.latlng.lng}`;
    if (marker) marker.setLatLng(e.latlng);
}

function addTag(tag){
    var list = element('taglist');
    if (selectedTags.size<1) list.innerHTML = '';
    list.innerHTML += `<span><button onclick="this.parentNode.parentNode.removeChild(this.parentNode);" title="click to remove">${tag}</button> </span>`;
    selectedTags.add(tag);
    var btn = element('save');
    btn.removeAttribute("disabled");
    btn.onclick= function(){ saveEvent(); };
    var select = element('proposals');
    select.innerHTML = '';
    select.style.display = 'none';
    var input = element('tags-input')
    input.value = '';
    input.focus();

}

function selectKeyPress(e){
    if (e.keyCode == 13){
        addTag(e.target.value);
    }
}

function tagKeyPress(e){
    var input = e.target;
    if (e.keyCode == 13){
        var tag = input.value;
        if (!tag) return;
        addTag(tag);

    } else if (e.keyCode == 40){
        var select = element('proposals');
        var val = select.firstChild.value;
        select.value = val;
        select.focus();
    } else {
        if (keyTimer != null) clearTimeout(keyTimer);
        setTimeout(() => fetchTags(input.value),500);
    }
}

function getTags(){
    var list = element('taglist');
    var buttons = list.getElementsByTagName('button');
    var tags = [];
    for(let i = 0;i < buttons.length; i++){
        tags.push(buttons[i].innerHTML);
    }
    return tags;
}

function getLinks(){
   var list = element('link-list');
   var anchors = list.getElementsByTagName('a');
   var links = [];
   for(let i = 0;i < anchors.length; i++){
        var anchor = anchors[i];
        if (anchor.href) links.push({description:anchor.innerHTML,url:anchor.href});
   }
   return links;
}

function getAttachments(){
   var list = element('attachment-list');
   var images = list.getElementsByTagName('img');
   var urls = [];
   for(let i = 0;i < images.length; i++){
        var img = images[i];
        if (img.src) urls.push(img.src);
   }
   return urls;
}


function showError(message){
    var span = document.createElement('div');
    span.setAttribute('class','error');
    span.innerHTML = "<span>Error: "+message+"<span>";
    span.style.opacity = '1';
    span.style.transition = 'opacity 1000ms linear';
    element('taglist').appendChild(span);
    setTimeout(() => {
        span.style.opacity = '0';
        setTimeout(() => span.parentNode.removeChild(span),1000);
    },4000);
}


async function handleSave(response){
    if (response.ok){
        var json = await response.json();
        var url = window.location.href.replace('/static/edit','')
        let pos = url.indexOf('?');
        if (pos>0) url = url.substring(0,pos);
        if (json.id) url+='?id='+json.id;
        window.location.href = url;
    } else {
        var json = await response.json();
        if (json.error) showError(json.error);
    }
}

async function saveEvent(){
    element('save').toggleAttribute("disabled");

    var location = element('location').value;
    var start = element('start').value;
    var aid = element('aid').value;
    var event = {
        aid : aid ? aid : 0,
        title : element('title').value,
        description : element('description').value,
        location : element('location').value,
        start : element('start').value,
        end : element('end').value,
        tags: getTags(),
        links: getLinks(),
        coords: element('coords').value,
        attachments: getAttachments()
    };
    fetch('/api/event',{
        method: aid ? 'PATCH' : 'POST', // if aid is set, we do an update. otherwise save new
        body: JSON.stringify(event),
        headers: {
            'Content-Type' : 'appication/json',
            'title': event.title
        }
    }).then(handleSave);
}

async function distributeEventData(response){
    if (response.ok){
        let event = await response.json();
        console.log({'event-data':event});
        var keys = ['title','location','description'];
        if (element('aid').value) {
            keys.push('start');
            keys.push('end');
        }
        for (var key of keys){
            if (event[key]) element(key).value = event[key];
        }
        if (event.coords){
            element('coords').value = event.coords.lat + ', ' + event.coords.lon;
            marker.setLatLng([event.coords.lat, event.coords.lon]);
            map.setView([event.coords.lat, event.coords.lon], 17);
        }


        var linkList = element('link-list');

        for (var link of event.links){
            var span = document.createElement('span');
            span.innerHTML = `<a href="${link.url}">${link.description}</a><a onclick="removeParent(this)">❌</a> `;
            linkList.appendChild(span);
        }

        var imgList = element('attachment-list');
        for (var image of event.attachments){
            var span = document.createElement('span');
            span.innerHTML = `<img src="${image.url}" /><a onclick="removeParent(this)">❌</a> `;
            imgList.appendChild(span);
        }

        if (event.tags){
            for (var tag of event.tags) addTag(tag);
        }
    }
}

function loadEvent(){
    let params = new URLSearchParams(window.location.search);
    var aid = params.get('id');
    if (aid){
        element('aid').value = aid;
    } else aid = params.get('copy');
    if (aid) {
        fetch('/api/event?id='+aid).then(distributeEventData);
    } else {
        const urlParams = new URLSearchParams(window.location.search);
        var tags = urlParams.get('tags');}
        if (tags){
            tags = tags.split(',');
            for (var tag of tags) addTag(tag);
        }
    }