/* © SRSoftware 2024 */
import static de.srsoftware.cal.ApiEndpoint.parseDate;
import static java.time.LocalDateTime.of;
import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Test;

public class ParseDateTest {
	@Test
	public void testFull() {
		assertEquals(of(2024, 12, 11, 10, 9, 8), parseDate("2024-12-11T10:09:08").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 10, 9, 8), parseDate("2024-12-11 10:09:08").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 10, 9, 8), parseDate("2024-12-11T10:9:8").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 10, 9, 8), parseDate("2024-12-11 10:9:8").optional().orElse(null));
	}

	@Test
	public void testNoSecond() {
		assertEquals(of(2024, 12, 11, 10, 9), parseDate("2024-12-11T10:09").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 10, 9), parseDate("2024-12-11 10:09").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 10, 9), parseDate("2024-12-11T10:9").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 8, 9), parseDate("2024-12-11 8:9").optional().orElse(null));
	}

	@Test
	public void testNoTime() {
		assertEquals(of(2024, 12, 11, 0, 0), parseDate("2024-12-11").optional().orElse(null));
		assertEquals(of(2024, 12, 11, 0, 0), parseDate("2024-12-11").optional().orElse(null));
		assertEquals(of(2024, 12, 5, 0, 0), parseDate("2024-12-5").optional().orElse(null));
		assertEquals(of(2024, 12, 5, 0, 0), parseDate("2024-12-5").optional().orElse(null));
		assertEquals(of(2024, 6, 5, 0, 0), parseDate("2024-6-5").optional().orElse(null));
		assertEquals(of(2024, 6, 5, 0, 0), parseDate("2024-6-5").optional().orElse(null));
	}

	@Test
	public void testNoDay() {
		assertEquals(of(2024, 12, 1, 0, 0), parseDate("2024-12").optional().orElse(null));
		assertEquals(of(2024, 12, 1, 0, 0), parseDate("2024-12").optional().orElse(null));
	}
}
